/********************************************************************
  BasicDSP
  Copyright 2006-2007 
  Pieter-Tjerk de Boer PA3FWM pa3fwm@amsat.org
  Niels Moseley PE1OIT n.a.moseley@alumnus.utwente.nl
  License: GPLv2
********************************************************************/

#include "book.h"


IMPLEMENT_DYNAMIC_CLASS(wxBook, wxControl)
BEGIN_EVENT_TABLE(wxBook, wxBookCtrlBase)
    EVT_SIZE(wxBook::OnSize)
    //EVT_CHOICE(wxID_CHOICEBOOKCHOICE, wxBook::OnChoiceSelected)
END_EVENT_TABLE()


wxBook::wxBook(wxWindow *parent,
                 wxWindowID id,
                 const wxPoint& pos,
                 const wxSize& size,
                 long style) : m_selected(wxNOT_FOUND)
{
  wxControl::Create(parent, id, pos, size, style,
                            wxDefaultValidator, _T(""));
}

wxBook::~wxBook() 
{
}

int wxBook::SetSelection(size_t n)
{
  int old_selected = m_selected;

  if (n >= m_pages.size()) return old_selected;   //actually an error
  

  if ((int)n != m_selected)
  {
    if (m_selected != wxNOT_FOUND) m_pages[m_selected]->Hide();
    m_selected = n;
    wxWindow *page = m_pages[n];
    page->SetSize(GetPageRect());
    page->Show();
  }
  return old_selected;
}

wxWindow* wxBook::DoRemovePage(size_t page)
{
  wxWindow *win = wxBookCtrlBase::DoRemovePage(page);
  return win;
}

wxSize wxBook::CalcSizeFromPage(const wxSize& sizePage) const
{
  // we have no internal selection controls.
  // therefore, the control is a big as the page itself!
  return sizePage;
}

void wxBook::OnSize(wxSizeEvent& event)
{
    event.Skip();

    if ((m_pages.size()==0) || (m_selected == wxNOT_FOUND)) return;

    // resize the currently shown page
    if (m_selected != wxNOT_FOUND)
    {
        wxWindow *page = m_pages[m_selected];
        wxCHECK_RET(page, _T("NULL page in wxChoicebook?") );
        page->SetSize(GetPageRect());
    }
}

wxRect wxBook::GetPageRect() const
{
  wxPoint pt;
  wxRect rectPage(pt, GetClientSize());
  return rectPage;
}

bool wxBook::InsertPage(size_t n,
                          wxWindow *page,
                          const wxString &text,
                          bool bSelect,
                          int imageId)
{
  if (!wxBookCtrlBase::InsertPage(n, page, text, bSelect, imageId))
        return false;

  if ((bSelect) || (m_selected == wxNOT_FOUND))
    SetSelection(n);
  else 
    m_pages[n]->Hide();  // hide a new page, if it's not selected.

  return true;
}
