/********************************************************************
  BasicDSP
  Copyright 2006-2007 
  Pieter-Tjerk de Boer PA3FWM pa3fwm@amsat.org
  Niels Moseley PE1OIT n.a.moseley@alumnus.utwente.nl
  License: GPLv2
********************************************************************/

#include <wx/wx.h>
#include <wx/event.h>
#include "displayslider.h"

DEFINE_EVENT_TYPE(wxEVT_COMMAND_DISPLAYSLIDER_EVENT)


DisplaySlider::DisplaySlider(wxWindow *parent, wxSizer *_parentsizer, const wxString &_title, wxWindowID sliderID,
                             float value, float slidermin, float slidermax, const wxString& format, int bordersize)
                             : myparent(parent), parentsizer(_parentsizer), slider_ID(sliderID), 
                             formatstring(format), vmin(slidermin), vmax(slidermax), isHidden(false)
{

  hsizer = new wxBoxSizer(wxHORIZONTAL);
  slider = new wxSlider(parent, -1, 0, 0, 10000, wxDefaultPosition, wxDefaultSize, wxHORIZONTAL);  
  
  title = new wxStaticText(parent, -1, _title);

  text = new wxTextCtrl(parent, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_DONTWRAP | wxTE_RICH | wxTE_PROCESS_ENTER);
  text->SetMaxLength(10);
  
  hsizer->Add(title, 0, wxALL, 4);
  hsizer->Add(slider, 1, wxEXPAND);
  hsizer->Add(text, 0, wxALL, 4);
  parentsizer->Add(hsizer, 1, wxEXPAND | wxALL, bordersize);

  slider->Connect(wxEVT_SCROLL_THUMBTRACK, wxScrollEventHandler(DisplaySlider::OnSlider),NULL,this);
  text->Connect(wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler(DisplaySlider::OnSliderText),NULL,this);

  SetValue(value); // do this after creating the text control, it uses it!
}

DisplaySlider::~DisplaySlider()
{
}

bool DisplaySlider::Hide()
{
  //remove the hsizer from the parentsizer
  if (!isHidden)
  { 
    // the following is a work-around to keep wxWidgets
    // from resizing a hidden slider
    wxSize myminsize = hsizer->GetSize();
    hsizer->SetMinSize(myminsize);

    slider->Show(false);
    text->Show(false);
    isHidden = true;
    return true;
  }
  return false;
}

bool DisplaySlider::Show()
{
  if (isHidden) 
  { 
    slider->Show(true);
    text->Show(true);
    hsizer->Layout();
    isHidden = false;
    return true;
  }
  return false;
}

void DisplaySlider::SetMinTitleWidth(int width)
{
  wxSize titlesize = title->GetSize();
  titlesize.SetWidth(width);
  title->SetMinSize(titlesize);
  title->SetSize(titlesize);
}

int DisplaySlider::GetTitleWidth() const
{
  wxSize mysize = title->GetSize();
  return mysize.GetWidth();
}

float DisplaySlider::GetValue()
{
  int v = slider->GetValue();
  float range = vmax - vmin;
  return vmin + (range*static_cast<float>(v)/10000.0f);
}

void DisplaySlider::SetValue(float value)
{
  float range = vmax - vmin;
  int v = (int)(10000.0f*(value - vmin)/range);
  if (v < 0) v = 0;
  if (v > 10000) v = 10000;
  slider->SetValue(v);
  UpdateDisplay();
  hasChanged = true;
}

void DisplaySlider::OnSlider(wxScrollEvent& event)
{
   UpdateDisplay();
   wxCommandEvent event2 ( wxEVT_COMMAND_DISPLAYSLIDER_EVENT, slider_ID );
   event2.SetEventObject(this);
   slider->GetEventHandler()->ProcessEvent( event2 );
   hasChanged = true;
}

void DisplaySlider::OnSliderText(wxCommandEvent& event)
{
   double v;
   if (!event.GetString().BeforeFirst(' ').ToDouble(&v)) return;  // the BeforeFirst eliminates things like " Hz" 
   SetValue(v);
   wxCommandEvent event2 ( wxEVT_COMMAND_DISPLAYSLIDER_EVENT, slider_ID );
   event2.SetEventObject(this);
   slider->GetEventHandler()->ProcessEvent( event2 );
}


void DisplaySlider::UpdateDisplay()
{
  displaytext.Printf(formatstring,GetValue());
  text->SetValue(displaytext);
}

bool DisplaySlider::HasChanged()
{
  if (hasChanged)
  {
    hasChanged = false;
    return true;
  }
  return false;
}
