/********************************************************************
  BasicDSP
  Copyright 2006-2007 
  Pieter-Tjerk de Boer PA3FWM pa3fwm@amsat.org
  Niels Moseley PE1OIT n.a.moseley@alumnus.utwente.nl
  License: GPLv2
********************************************************************/

#include "gui.h"
#include "help.h"

#include <wx/cmdline.h>

enum
{
   ID_Slider = 1000,       // actually + slidernumber
   ID_Quit = 1, 
   ID_About,
   ID_HelpVariables, ID_HelpFunctions, ID_HelpIntro,
   ID_HelpHomepage1, ID_HelpHomepage2,
   ID_Timer,
   ID_Open, ID_Reload, ID_Save, ID_SaveAs, ID_Clear, ID_ShowSpectrum, ID_ShowOscilloscope,
   ID_Runbutton, ID_Stopbutton,
   ID_Textchanged,
   ID_OpenWavFile,
   ID_InputRadioGroup,
   ID_DCOffset
};


extern bool remove_dc;  // this flag is in core.cpp

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
   EVT_MENU(ID_Quit, MyFrame::OnQuit)
   EVT_MENU(ID_About, MyFrame::OnAbout)
   EVT_MENU(ID_HelpIntro, MyFrame::OnHelp)
   EVT_MENU(ID_HelpFunctions, MyFrame::OnHelp)
   EVT_MENU(ID_HelpVariables, MyFrame::OnHelp)
   EVT_MENU(ID_HelpIntro, MyFrame::OnHelp)
   EVT_MENU(ID_HelpHomepage1, MyFrame::OnHelpHomepage1)
   EVT_MENU(ID_HelpHomepage2, MyFrame::OnHelpHomepage2)
   EVT_MENU(ID_Open, MyFrame::OnOpen)
   EVT_MENU(ID_Reload, MyFrame::OnReload)
   EVT_MENU(ID_Save, MyFrame::OnSave)
   EVT_MENU(ID_SaveAs, MyFrame::OnSaveAs)
   EVT_MENU(ID_Clear, MyFrame::OnClear)
   EVT_BUTTON(ID_OpenWavFile, MyFrame::OnOpenWaveFile)
   EVT_TIMER(ID_Timer, MyFrame::OnTimer)
   EVT_BUTTON(ID_Runbutton, MyFrame::OnRunbutton)
   EVT_BUTTON(ID_Stopbutton, MyFrame::OnStopbutton)
   EVT_TOGGLEBUTTON(ID_ShowSpectrum, MyFrame::OnShowSpectrum)
   EVT_TOGGLEBUTTON(ID_ShowOscilloscope, MyFrame::OnShowOscilloscope)
   EVT_TEXT(ID_Textchanged, MyFrame::OnTextchanged)
   EVT_RADIOBOX(ID_InputRadioGroup, MyFrame::OnInputChanged)
   EVT_CHECKBOX(ID_DCOffset, MyFrame::OnDCOffsetChanged)
END_EVENT_TABLE()

IMPLEMENT_APP(MyApp)

// indexes of the checkboxes in specframe and scopeframe
#define CH_100dB 0
#define CH_Average 1
#define CH_Quadrature 2
#define CH_Freeze 0
#define CH_Xzoom 1
#define CH_Yzoom 2


// ---------------------------------------------------------------

static const wxCmdLineEntryDesc cmdLineDesc[] =
{
   { wxCMD_LINE_SWITCH, _T("s"), _T("sine"), _T("use sine as input source") },
   { wxCMD_LINE_SWITCH, _T("n"), _T("noise"),_T("use noise as input source") },
   { wxCMD_LINE_SWITCH, NULL, _T("quadsine"),_T("use quadsine as input source") },
   { wxCMD_LINE_OPTION, _T("r"), _T("samplerate"),_T("set samplerate"), wxCMD_LINE_VAL_NUMBER},
   { wxCMD_LINE_OPTION, _T("f"), _T("fontsize"), _T("set font size"), wxCMD_LINE_VAL_NUMBER },
   { wxCMD_LINE_PARAM,  NULL, NULL, _T("input file"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_MULTIPLE | wxCMD_LINE_PARAM_OPTIONAL},
   { wxCMD_LINE_NONE }
};

bool MyApp::OnInit()
{
   wxCmdLineParser cmdparser;
   long value;
   cmdparser.SetDesc(cmdLineDesc);
   cmdparser.SetLogo(APPNAME _T(" ") VERSION _T(" by Pieter-Tjerk de Boer PA3FWM and Niels Moseley PE1OIT\n"));
   cmdparser.SetCmdLine(argc, argv);
   if (cmdparser.Parse(true) != 0) return false;  // if the parser encounters an error, a usage text will be shown.

   int fontsize=0;
   if (cmdparser.Found(_T("fontsize"), &value)) fontsize=value;

   MyFrame *frame = new MyFrame(APPNAME, wxDefaultPosition, wxDefaultSize, fontsize );
   frame->Show(TRUE);
   SetTopWindow(frame);

   frame->setsource(INPUT_SOUND);
   if (cmdparser.Found(_T("sine"))) frame->setsource(INPUT_SINE);
   if (cmdparser.Found(_T("noise"))) frame->setsource(INPUT_NOISE);
   if (cmdparser.Found(_T("quadsine"))) frame->setsource(INPUT_QUADSINE);

   const size_t params = cmdparser.GetParamCount();
   for(size_t i=0; i<params; i++)
   {
      wxString param(cmdparser.GetParam(i));
      // check if the parameter ends in .wav
      // if so, set the input to WAV and load the WAV file
      // else assume it is a script file and load it into the editor
      wxString param_ends(param.Right(4));
      if (param_ends.CmpNoCase(_T(".wav")) == 0)
      {
         frame->DoOpenWaveFile(param);
         frame->setsource(INPUT_WAVFILE);
      }
      else
      {
         frame->loadfile(param);
      }
   }

   if (cmdparser.Found(_T("samplerate"), &value)) virtsamplerate = static_cast<float>(value);

   corethread = new CoreThread();
   corethread->Create();
   corethread->Run();
    
   return TRUE;
} 

int MyApp::OnExit()
{
#ifdef WIN32
  // gracefully terminate the core thread,
  // otherwise we'll get access errors
  quit_flag = true;       // signal the thread to exit

  if (corethread!=NULL)
    corethread->Wait();   // wait for the thread to join the main thread.
#else
  //FIXME: signal the linux thread to join the main thread
  //then call corethread->Wait();
  //this will wait until the corethread has joined
  //then exit (don't kill - it's cruel!)
  corethread->Kill();     // simply kill the thread in Linux.
#endif

  return wxApp::OnExit();
}

// ---------------------------------------------------------------

MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size, int fontsize)
: wxFrame((wxFrame *)NULL, -1, title, pos, size),
  textfont(wxNORMAL_FONT->GetPointSize(), wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL),
  specframe(NULL), scopeframe(NULL),
  m_timer(this, ID_Timer)
{
   wxMenu *menuFile = new wxMenu;
   wxMenu *menuHelp = new wxMenu;
   panel = new wxPanel(this);
   topsizer = new wxBoxSizer(wxVERTICAL);
   wxBoxSizer *sizer;
   wxBoxSizer *sizer2;

   wxString stringbuf;  // buffer to build temporary strings 

   menuFile->Append( ID_Open, _T("&Open"));
   menuFile->Append( ID_Reload, _T("&Reload"));
   menuFile->Append( ID_Save, _T("&Save"));
   menuFile->Append( ID_SaveAs, _T("Save &As"));
   menuFile->Append( ID_Clear, _T("&Clear"));
   menuFile->AppendSeparator();
   menuFile->Append( ID_Quit, _T("E&xit"));
  
   menuHelp->Append( ID_HelpIntro,     _T("&Introduction"));
   menuHelp->Append( ID_HelpVariables, _T("&Variables help"));
   menuHelp->Append( ID_HelpFunctions, _T("&Functions help"));
   
#ifdef WIN32
   menuHelp->AppendSeparator();
   menuHelp->Append(ID_HelpHomepage1, _T("BasicDSP homepage PA3FWM"));
   menuHelp->Append(ID_HelpHomepage2, _T("Homepage PE1OIT"));   
#endif
   menuHelp->AppendSeparator();
   menuHelp->Append( ID_About, _T("About"));    //removed 'A' as accelerator - it is already allocated to Save &As.

   wxMenuBar *menuBar = new wxMenuBar();
   menuBar->Append( menuFile, _T("&File"));
   menuBar->Append( menuHelp, _T("&Help"));

   SetMenuBar( menuBar );

   text = new wxTextCtrl(panel, ID_Textchanged, _T(""), wxDefaultPosition, wxSize(-1,200),
          wxTE_MULTILINE | wxTE_DONTWRAP | wxTE_RICH2);
   // set text to 'modern' to force a nonproportional font
   if (fontsize>0) textfont.SetPointSize(fontsize);
   text->SetFont(textfont);
   topsizer->Add(text, 1, wxEXPAND | wxALL, 4);

   sizer = new wxBoxSizer(wxHORIZONTAL);
   sizer->Add( new wxButton(panel, ID_Runbutton, _T("&Run")), 0, wxALL, 4);
   sizer->Add( new wxButton(panel, ID_Stopbutton, _T("Stop")), 0, wxALL, 4);
   statustext = new wxStaticText(panel, -1, _T(""));
   sizer->Add( statustext, 1, wxEXPAND | wxALL, 4);
   oscilloscopebutton = new wxToggleButton(panel, ID_ShowOscilloscope, _T("Scope"));
   sizer->Add(oscilloscopebutton , 0, wxALL, 4);
   spectrumbutton = new wxToggleButton(panel, ID_ShowSpectrum, _T("Spectrum"));
   sizer->Add(spectrumbutton , 0, wxALL, 4);
   topsizer->Add(sizer, 0, wxEXPAND);

   sizer2 = new wxStaticBoxSizer( new wxStaticBox(panel, -1, _T("")), wxVERTICAL);
      // add wxRadioBox control
      wxArrayString choices;

      // add wxBook control
      inputbook = new wxBook(panel, -1, wxDefaultPosition, wxDefaultSize, wxSTATIC_BORDER);
      wxWindow *panel_owner = inputbook;
      
      // Soundcard page
      wxPanel *page = new wxPanel(panel_owner);
        sizer = new wxBoxSizer(wxVERTICAL);
        removedc = new wxCheckBox(page, ID_DCOffset, _T("Remove DC offset"));
        sizer->Add(removedc, 0, wxALL, 6);
        removedc->SetValue(true);
        page->SetSizer(sizer);
      inputbook->AddPage(page, _T("Soundcard"), true);
      choices.Add(_T("Soundcard"));
      inputidentifiers.push_back(INPUT_SOUND);
      
      // Noise page
      page = new wxPanel(panel_owner);
      inputbook->AddPage(page, _T("Noise"));
      choices.Add(_T("Noise"));
      inputidentifiers.push_back(INPUT_NOISE);

      // Sine page
      page = new wxPanel(panel_owner);
        sizer = new wxBoxSizer(wxVERTICAL);
        DisplaySlider *freqslider = new DisplaySlider(page, sizer, _T("Frequency"), ID_Slider, 0.1f*virtsamplerate/2, 0.0f, virtsamplerate/2, _T("%.1f Hz"), 4);
        Connect(ID_Slider,wxEVT_COMMAND_DISPLAYSLIDER_EVENT, wxCommandEventHandler(MyFrame::OnDisplaysliderEvent),NULL,this);
        freqsliders.push_back(freqslider);
        page->SetSizer(sizer);
      inputbook->AddPage(page, _T("Sine"));
      choices.Add(_T("Sine"));
      inputidentifiers.push_back(INPUT_SINE);

      // Quadsine page
      page = new wxPanel(panel_owner);
        sizer = new wxBoxSizer(wxVERTICAL);
        freqslider = new DisplaySlider(page, sizer, _T("Frequency"), ID_Slider, 0.0f, -virtsamplerate/2, virtsamplerate/2, _T("%.1f Hz"), 4);
        Connect(ID_Slider,wxEVT_COMMAND_DISPLAYSLIDER_EVENT, wxCommandEventHandler(MyFrame::OnDisplaysliderEvent),NULL,this);
        freqsliders.push_back(freqslider);
        page->SetSizer(sizer);
      inputbook->AddPage(page, _T("Quadsine"));
      choices.Add(_T("Quadsine"));
      inputidentifiers.push_back(INPUT_QUADSINE);

      // Wave file page  
      page = new wxPanel(panel_owner);
        sizer = new wxBoxSizer(wxHORIZONTAL);
        wxButton *button_wavefileopen = new wxButton(page, ID_OpenWavFile, _T("Open wave file"));
        // create static text to show loaded wave file properties
        // make the string 3 lines high -- this will force the correct size of the control!
        waveinfotext = new wxStaticText(page, -1, _T("No wave file loaded...\n\n"), wxDefaultPosition, 
          wxDefaultSize);
        waveinfotext->Wrap(-1); //disable text wrapping -- I don't like 50cent anyway.
        sizer->Add(button_wavefileopen, 0, wxALL, 8);
        sizer->Add(waveinfotext, 1, wxALL, 8);
        page->SetSizer(sizer);      
      inputbook->AddPage(page, _T("WAV file"));
      choices.Add(_T("WAV file"));
      inputidentifiers.push_back(INPUT_WAVFILE);

      // Impulse page
      page = new wxPanel(panel_owner);
      inputbook->AddPage(page, _T("Impulse"));
      choices.Add(_T("Impulse"));
      inputidentifiers.push_back(INPUT_IMPULSE);

      // add the input selector
      inputselector = new wxRadioBox(panel, ID_InputRadioGroup, _T("Input source"), wxDefaultPosition, 
        wxDefaultSize, choices, 0, wxHORIZONTAL);
      sizer2->Add(inputselector, 0, wxEXPAND | wxALL, 4);

      // end of wxBook
      sizer2->Add(inputbook, 0, wxEXPAND | wxALL, 6);
   topsizer->Add(sizer2, 0, wxEXPAND | wxALL, 8);

   // add the parameter sliders
   for(int i=1;i<NUMSLIDERS;i++) 
   {
      wxString title(_T("Slider "));
      sizer = new wxStaticBoxSizer( new wxStaticBox(panel, -1, stringbuf), wxHORIZONTAL);
      title << i;
      DisplaySlider *s = new DisplaySlider(panel, sizer, title, ID_Slider+i, 0.1f*i, 0.0f, 1.0f, _T("%g"));
      paramsliders.push_back(s);

      // connect the event handlers to the IDs
      // this saves a lot of hard-coded event table stuff.
      Connect(ID_Slider+i,wxEVT_COMMAND_DISPLAYSLIDER_EVENT, wxCommandEventHandler(MyFrame::OnDisplaysliderEvent),NULL,this);
      topsizer->Add(sizer, 0, wxEXPAND);
   }

   // equalize the slider title width
   // this takes care of asymmetric 
   // find max
   int title_width_max = -1;
   for(int i=1;i<NUMSLIDERS;i++)
   {
     int w = paramsliders[i-1]->GetTitleWidth();
     if (w > title_width_max) title_width_max = w;
   }
   for(int i=1;i<NUMSLIDERS;i++)
   {
     paramsliders[i-1]->SetMinTitleWidth(title_width_max);
   }

   // add the VU meters
   vu_left = new wxVUMeter(panel, -1, _T("Left"));
   vu_right = new wxVUMeter(panel, -1, _T("Right"));
   topsizer->Add(vu_left,  0, wxEXPAND | wxALL, 4);
   topsizer->Add(vu_right, 0, wxEXPAND | wxALL, 4);

   panel->SetSizer(topsizer);
   topsizer->SetSizeHints(panel);

   Fit();

//   m_timer.Start(20);  // 20 ms interval
   m_timer.Start(40);  // 40 ms interval

   filename=_T("");
}

void MyFrame::UpdateWindowSize()
{
  Layout();
  Refresh();
}

void MyFrame::OnTimer(wxTimerEvent& event)
{
   int i;
   wxString stringbuf;
   static float prevvirtsamplerate=0;

   // if samplerate has changed, update the frequency ranges on the sine and quadsine sliders
   if (prevvirtsamplerate!=virtsamplerate) {
      freqsliders[0]->SetSliderMax(virtsamplerate/2);
      freqsliders[0]->UpdateDisplay();
      freqsliders[1]->SetSliderMin(-virtsamplerate/2);
      freqsliders[1]->SetSliderMax(virtsamplerate/2);
      freqsliders[1]->UpdateDisplay();
      if (inputsource==INPUT_QUADSINE) setslider_core( 0, freqsliders[1]->GetValue() );
      else setslider_core( 0, freqsliders[0]->GetValue() );
      prevvirtsamplerate=virtsamplerate;
   }

   // update VU meters
   float l_max = left_channel_max;
   float r_max = right_channel_max;
   vu_left->SetLevel(l_max);
   vu_right->SetLevel(r_max);

   // check for WIN32 audio error
#ifdef WIN32
	if (AudioError[0]!=0)
	{
		if (statustext!=NULL) 
    {
      wxString str;
      str.FromAscii(AudioError);
      statustext->SetLabel(str);
    }
		statustext->Refresh();
		AudioError[0] = 0;
	}
#endif

   // update spectrum plot, if that window is open
   if (specframe) {
      static int ip[2+64]={0};
      static float w[NFFT/2];
      static float window[NFFT];
      static float window2[NFFT/2];
      float dl[NFFT];
      float el[NFFT/2];
      float dr[NFFT];
      float er[NFFT/2];
      float a,b;
      if (ip[0]==0) {
         // first time: initialize the arrays containing the window coefficients
         for (i=0;i<NFFT/2;i++) {
            float j = ((NFFT+1.0)/2 - i) / NFFT;
            window[i] = window[NFFT-1-i] =
               ( 0.35875 + 0.48829 * cos (2 * M_PI * j) + 0.14128 * cos (4 * M_PI * j) + 0.01168 * cos (6 * M_PI * j) ) ;
            // Blackman-Harris window
         }
         for (i=0;i<NFFT/4;i++) {
            float j = ((NFFT/2+1.0)/2 - i) / (NFFT/2);
            window2[i] = window2[NFFT/2-1-i] =
               ( 0.35875 + 0.48829 * cos (2 * M_PI * j) + 0.14128 * cos (4 * M_PI * j) + 0.01168 * cos (6 * M_PI * j) ) ;
            // Blackman-Harris window
         }
      }
//      if (!specframe->Quadrature() || inputsource==INPUT_IMPULSE) {
      if (!specframe->check(CH_Quadrature) || inputsource==INPUT_IMPULSE) {
         // Quadrature disabled: do separate real FFTs for left and right channel
         // no support yet for quadrature impulse response
         if (inputsource!=INPUT_IMPULSE) {
            for (i=0;i<NFFT;i++) {
               int j = (var_to_gui_idx - NFFT + i) & (VAR_TO_GUI_LEN-1);
               dl[i] = var_to_gui[0][j] /NFFT*4*1.414;  // TODO: check this scaling factor other than experimentally
               dr[i] = var_to_gui[1][j] /NFFT*4*1.414;
               dl[i] *= window[i];
               dr[i] *= window[i];
            }
         } else {
            int l = (var_to_gui_idx_prevpulse-var_to_gui_idx_prevprevpulse) & (VAR_TO_GUI_LEN-1);
            if (l>NFFT) l=NFFT;
            for (i=0; i<l; i++) {
               int j = (var_to_gui_idx_prevprevpulse + i) & (VAR_TO_GUI_LEN-1);
               dl[i] = var_to_gui[0][j];
               dr[i] = var_to_gui[1][j];
            }
            for (i=l; i<NFFT; i++) dl[i]=dr[i]=0;
         }
         rdft(NFFT, 1, dl, ip, w);
         rdft(NFFT, 1, dr, ip, w);
         el[0]=dl[0]*dl[0];
         er[0]=dr[0]*dr[0];
         for (i=1;i<NFFT/2;i++) {
            el[i] = dl[2*i]*dl[2*i] + dl[2*i+1]*dl[2*i+1];
            er[i] = dr[2*i]*dr[2*i] + dr[2*i+1]*dr[2*i+1];
         }
      } else {
         // Quadrature enabled: do a single complex FFT
         for (i=0;i<NFFT/2;i++) {
            int j = (var_to_gui_idx - NFFT/2 + i) & (VAR_TO_GUI_LEN-1);
            dl[2*i] = var_to_gui[1][j] /NFFT*4*1.414;
            dl[2*i] *= window2[i];
            dl[2*i+1] = var_to_gui[0][j] /NFFT*4*1.414;
            dl[2*i+1] *= window2[i];
         }
         cdft(NFFT, 1, dl, ip, w);
         er[NFFT/4]= el[NFFT/4] = dl[0]*dl[0];
         for (i=1;i<NFFT/2;i++) {
            er[i^(NFFT/4)] = el[i^(NFFT/4)] = dl[2*i]*dl[2*i] + dl[2*i+1]*dl[2*i+1];
         }
      }
      // final calculations: averaging if desired, noise level correction, and logarithmic scale
      if (specframe->check(CH_Average)) a=0.02; else a=1;
      b=0;
      if (inputsource==INPUT_NOISE) b=39.5;  // TODO: actually calculate this offset
      for (i=0;i<NFFT/2;i++) {
         spec_l[i] = a*el[i] + (1-a)*spec_l[i] + a*1e-20;  // the 1e-20 prevents trouble if el[i] happens to be exactly 0
         el[i] = 10*log10(spec_l[i])+b;
         spec_r[i] = a*er[i] + (1-a)*spec_r[i] + a*1e-20;
         er[i] = 10*log10(spec_r[i])+b;
      }
      // send the data to the specframe
      specframe->SetMinMax(
         specframe->check(CH_100dB) ? -100 : -50 ,
         5,
         specframe->check(CH_Quadrature) ? -virtsamplerate/2 : 0 ,
         virtsamplerate/2 );
      specframe->SetLevel(el,er);
      var_to_gui_setvar(0, specframe->var_l().ToAscii());
      var_to_gui_setvar(1, specframe->var_r().ToAscii());
   }

   // update oscilloscope, if that window is open
   if (scopeframe) {
      float el[NFFT/2];
      float er[NFFT/2];
      float tmax;
      float xzoom=1;
      if (inputsource!=INPUT_IMPULSE) {
         for (i=0;i<NFFT/2;i++) {
            int j = (var_to_gui_idx - NFFT/2 + i) & (VAR_TO_GUI_LEN-1);
            el[i] = var_to_gui[2][j];
            er[i] = var_to_gui[3][j];
         }
      } else {
         int l = (var_to_gui_idx_prevpulse-var_to_gui_idx_prevprevpulse) & (VAR_TO_GUI_LEN-1);
         if (l>NFFT/2) l=NFFT/2;
         for (i=0; i<l; i++) {
            int j = (var_to_gui_idx_prevprevpulse + i) & (VAR_TO_GUI_LEN-1);
            el[i] = var_to_gui[2][j];
            er[i] = var_to_gui[3][j];
         }
         for (i=l; i<NFFT/2; i++) el[i]=er[i]=0;
      }
      if (scopeframe->check(CH_Xzoom)) xzoom=10;
      tmax = NFFT/2/virtsamplerate*1000/xzoom;  // in ms
      if (scopeframe->check(CH_Yzoom)) scopeframe->SetMinMax(-0.22f,0.22f,0.0f,tmax,xzoom);
      else scopeframe->SetMinMax(-1.2f,1.2f,0.0f,tmax,xzoom);
      if (!scopeframe->check(CH_Freeze)) scopeframe->SetLevel(el,er);
      var_to_gui_setvar(2, scopeframe->var_l().ToAscii());
      var_to_gui_setvar(3, scopeframe->var_r().ToAscii());
   }
}


void MyFrame::updatetitle(void)   // changes the program's title to reflect a loaded file
{
   if (filename.Len() == 0) 
   {
      SetTitle(APPNAME);
   } 
   else 
   {
      wxString newtitle(APPNAME wxT(" - "));
      newtitle.Append(filename);
      SetTitle(newtitle);
   }
}


void MyFrame::settextbackgroundcolour(const wxColour& colour)
{
   text->SetStyle(0,text->GetLastPosition(), wxTextAttr(wxNullColour, colour, textfont));
   text->SetBackgroundColour(colour);
   text->RefreshRect(text->GetRect());
}

void MyFrame::loadfile(const wxString _filename)
{
   if (text->LoadFile(_filename)) 
   {
      filename=_filename;
      updatetitle();
      recompile();
   } 
   else 
   {
      wxMessageBox(_T("Load failed"), _T("Error"), wxOK | wxICON_INFORMATION, this);
   }
}

void MyFrame::OnInputChanged(wxCommandEvent& event)
{
   int index = inputselector->GetSelection();

   // change the inputbook to reflect the correct page
   inputbook->SetSelection(index);

   // convert the index (which is indexes the controls on the screen) to the appropriate input source identifier for the core code
   do_setsource(inputidentifiers[index]);
}

void MyFrame::setsource(int i)
{
   unsigned int j;
   do_setsource(i);
   for (j=0; j<inputidentifiers.size(); j++) if (inputidentifiers[j]==i) break;
   inputselector->SetSelection(j);
   inputbook->SetSelection(j);
}

void MyFrame::do_setsource(int i)
{
   inputsource=i;

   if (inputsource==INPUT_SOUND) remove_dc = removedc->GetValue();
   else remove_dc = false;

   if (inputsource==INPUT_SINE) setslider_core( 0, freqsliders[0]->GetValue() );
   if (inputsource==INPUT_QUADSINE) setslider_core( 0, freqsliders[1]->GetValue() );
}


void MyFrame::setslider_core(int i,float v)
{
   if (sliderptr[i]) *sliderptr[i]=v;
}


void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
   Close(TRUE);
}

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
  wxDialog *about = new AboutDialog(this, -1, _T("About BasicDSP"), wxDefaultPosition, wxDefaultSize);

  about->ShowModal();

  about->Destroy();
   //wxMessageBox(APPNAME _T(" version ") VERSION _T("\nCopyright 2006-2007, Pieter-Tjerk de Boer PA3FWM and Niels Moseley PE1OIT"),
   //   _T("About ") APPNAME, wxOK | wxICON_INFORMATION, this);
}


void MyFrame::OpenHelpWindow(const wxString &contents)
{
   wxFrame *frame = new wxFrame( this, -1, APPNAME _T(" Help"), wxDefaultPosition, wxDefaultSize );
   frame->Show(TRUE);
   wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
   wxHtmlWindow *w = new wxHtmlWindow(frame, -1, wxDefaultPosition, wxSize(500,500));
   sizer->Add(w, 1, wxEXPAND);
   w->SetPage(contents);
   frame->SetSizer(sizer);
   sizer->SetSizeHints(frame);
}
   

void MyFrame::OnHelp(wxCommandEvent& event)
{
   switch (event.GetId()) {
      case ID_HelpVariables: OpenHelpWindow(help_variables_html); break;
      case ID_HelpFunctions: OpenHelpWindow(help_functions_html); break;
      case ID_HelpIntro:     OpenHelpWindow(help_intro_html);     break;
   }
}


void MyFrame::OnHelpHomepage1(wxCommandEvent& WXUNUSED(event))
{
#ifdef WIN32
  ShellExecute(NULL, _T("open"), _T("http://wwwhome.cs.utwente.nl/~ptdeboer/ham/basicdsp/"), NULL, NULL, SW_SHOWNORMAL);
#endif
}

void MyFrame::OnHelpHomepage2(wxCommandEvent& WXUNUSED(event))
{
#ifdef WIN32
  ShellExecute(NULL, _T("open"), _T("http://home.tiscali.nl/curious_about/PE1OIT/"), NULL, NULL, SW_SHOWNORMAL);
#endif
}


void MyFrame::OnOpen(wxCommandEvent& WXUNUSED(event))
{
   wxFileDialog *fd = new wxFileDialog(this, _T("Open file"), _T(""), filename, _T("*.dsp"), wxOPEN|wxCHANGE_DIR|wxHIDE_READONLY);
   if (fd->ShowModal() == wxID_OK) loadfile(fd->GetPath());
   fd->Destroy();
}

void MyFrame::OnReload(wxCommandEvent& WXUNUSED(event))
{
   if (filename.Len()) loadfile(filename);
}

void MyFrame::OnSave(wxCommandEvent& event)
{
   if (filename.Len()==0) OnSaveAs(event);   // if the filename is empty, use the OnSaveAs procedure.
   else text->SaveFile(filename);
}

void MyFrame::OnSaveAs(wxCommandEvent& WXUNUSED(event))
{
   wxFileDialog *fd = new wxFileDialog(this, _T("Save file"), _T(""), filename, _T("*.dsp"), wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
   if (fd->ShowModal() == wxID_OK) {
      filename = fd->GetPath();
      text->SaveFile(filename);
      updatetitle();
   }
   fd->Destroy();
}

void MyFrame::OnClear(wxCommandEvent& WXUNUSED(event))
{
   backgroundgreen=0;
   settextbackgroundcolour(wxColour(wxT("WHITE")));
   text->SetValue(_T(""));
   filename=_T("");
   updatetitle();
}


void MyFrame::OnShowSpectrum(wxCommandEvent& WXUNUSED(event))
{
   if (specframe)
   {
      // Destroy the spectrum plot window
      specframe->Destroy();
      specframe = NULL;
      return;
   }
   
   // Show the spectrum plot window.
   static const wxChar* checknames[3] = {_T("100 dB"),_T("Average"),_T("Quadrature")};
   specframe = new MyGraphFrame(
      this,APPNAME _T("Spectrum"), 
      wxArrayString( 3, checknames ), _T("%d Hz"), _T("%d kHz"),
      this);
}


void MyFrame::OnShowOscilloscope(wxCommandEvent& WXUNUSED(event))
{
   if (scopeframe)
   {
      // Destroy the oscilloscope plot window
      scopeframe->Destroy();
      scopeframe = NULL;
      return;
   }
   
   // Show the oscilloscope plot window.
   static const wxChar* checknames[3] = {_T("Freeze"),_T("X zoom"),_T("Y zoom")};
   scopeframe = new MyGraphFrame(
      this, APPNAME _T("Oscilloscope"), 
      wxArrayString(3,checknames), _T("%d ms"), _T("%d s"),
      this);
}


void MyFrame::OnDisplaysliderEvent(wxCommandEvent &event)
{
   DisplaySlider *p=(DisplaySlider*)event.GetEventObject();
   setslider_core( event.GetId()-ID_Slider, p->GetValue() );
}


void MyFrame::OnOpenWaveFile(wxCommandEvent& WXUNUSED(event))
{
   wxFileDialog *fd = new wxFileDialog(this, _T("Open Wave file"), _T(""), filename, _T("*.wav"), wxOPEN|wxCHANGE_DIR|wxHIDE_READONLY);
   if (fd->ShowModal() == wxID_OK) DoOpenWaveFile(fd->GetPath());
   fd->Destroy();
}

void MyFrame::DoOpenWaveFile(const wxString& filename)
{
   int ret_val = 0;
   ret_val = wavstreamer.OpenFile(filename);
   if (ret_val == -2)
   {
    wxMessageBox(_T("Cannot open file."), _T("Error"), wxICON_INFORMATION);
   } 
   else if(ret_val == -1)
   {
    wxMessageBox(_T("File has invalid format. It should be in a PCM or IEEE floating-point data format and have two channels."), _T("Error"), wxICON_INFORMATION);
   }
   // put new information in the text control
   WavFormatChunk info;
   if (wavstreamer.GetFormat(info))
   {
     wxString str(_T("File: "));
     str << wavstreamer.GetFilename();
     str << _T("\n");
     str << _T("Format = ");
     switch(info.wFormatTag)
     {
     case 1:
       str << _T("PCM, ");
       break;
     case 3:
       str << _T("IEEE-FLOAT, ");
       break;
     }
//     str << _T("Bits per sample = ") << info.wBitsPerSample << _T("\n");
     str << info.wBitsPerSample << _T(" bits/sample") << _T("\n");
     str << _T("Sample rate = ") << info.dwSamplesPerSec << _T(" Hz");
     waveinfotext->SetLabel(str);
   }
}

void MyFrame::OnDCOffsetChanged(wxCommandEvent& event)
{
  remove_dc = removedc->GetValue();
}

void MyFrame::recompile(void)
{
   bool window_resize_needed = false;
   int i,l;
   int err=0;
   compile_start();
   backgroundgreen=0;
   settextbackgroundcolour(wxColour(wxT("WHITE")));
   l=text->GetNumberOfLines();
   for (i=0;i<l;i++) 
   {
      //FIXME: yikes - the following will fail if the script contains non-ASCII characters!
      //the best solution would be to change the parser/compiler to unicode compatible code
      //based on wxString etc.
      //-- hmm, I disagree, I'd rather keep the parser/compiler independent of the wx libraries, 
      //for reusability in non-wx projects.
      wxCharBuffer p_str = text->GetLineText(i).ToAscii();
      if ((p_str)[(size_t)0]>=' ') {
         // the above check is needed because (some) wxwidgets (version) erroneously returns a string starting with \n if an empty line is encountered.
         err = compile_one_line(p_str);
         if (err) {
            text->SetStyle(
               text->XYToPosition(erroroffset1,i),
               text->XYToPosition(erroroffset2,i),
               wxTextAttr(wxNullColour, wxColour(255,192,192), textfont));
            break;
         }
      }
   }
   compile_complete();
   for (i=1;i<NUMSLIDERS;i++)   // only the parameter sliders 
   {
      if (sliderptr[i]) 
      {
        if (paramsliders[i-1]->Show()) window_resize_needed = true;
        setslider_core(i, paramsliders[i-1]->GetValue());
      }  
      else
        if (paramsliders[i-1]->Hide()) window_resize_needed = true;
   }
   if (!err) {
      settextbackgroundcolour(wxColour(192,255,192));
      backgroundgreen=1;
      statustext->SetLabel(_T("OK"));
      statustext->SetBackgroundColour(wxColour(192,255,192));
   } 
   else 
   {
      wxString str;
      str = wxString::FromAscii(errorstr);
      statustext->SetLabel(str);
      statustext->SetBackgroundColour(wxColour(255,192,192));
      statustext->Refresh();
   }
   if (window_resize_needed) UpdateWindowSize();
   text->Refresh();    //Hmm.. we need to refresh for win32.
}

void MyFrame::OnRunbutton(wxCommandEvent& WXUNUSED(event))
{
   recompile();
}

void MyFrame::OnStopbutton(wxCommandEvent& WXUNUSED(event))
{
   core_stop();
}

void MyFrame::OnTextchanged(wxCommandEvent& WXUNUSED(event))
{
   if (backgroundgreen) {
      backgroundgreen=0;
      settextbackgroundcolour(wxColour(wxT("WHITE")));
      text->Refresh();    //Hmm.. we need to refresh for win32.
   }
}

void MyFrame::GFrameCloseNotify(MyGraphFrame *isClosing)
{
  if (isClosing == specframe) 
  {
    specframe = NULL;
    spectrumbutton->SetValue(false);
  }
  else if (isClosing == scopeframe)
  {
    scopeframe = NULL;
    oscilloscopebutton->SetValue(false);
  }
}


