/********************************************************************
  BasicDSP
  Copyright 2006-2007 
  Pieter-Tjerk de Boer PA3FWM pa3fwm@amsat.org
  Niels Moseley PE1OIT n.a.moseley@alumnus.utwente.nl
  License: GPLv2
********************************************************************/

#ifndef _gui_h
#define _gui_h

#include <iostream>
#include <sstream>
#include <vector>
#include <math.h>

#include "wx/wx.h" 
#include <wx/html/htmlwin.h>
#include <wx/tglbtn.h>

#include "aboutdialog.h"
#include "book.h"

#include <wx/textctrl.h>

#include "core.h"
#include "vu_meter.h" 
#include "spectrumplot.h" 
#include "displayslider.h"
#include "graphframe.h"
#include "fft4g.h"
      
#define VERSION _T("1.0")
#define APPNAME _T("BasicDSP")

#define NFFT 2048

#ifdef WIN32
extern char AudioError[1024];	// contains an error string -- if there is an audio error.
							                // contains 0 if no error
#include <windows.h>
#endif 

// ---------------------------------------------------------------

// class that encapsulates the audio & DSP process
class CoreThread: public wxThread
{
public:
   CoreThread() : wxThread(wxTHREAD_JOINABLE) {};
   virtual ~CoreThread() {};

   void *Entry()
   {
      coremain();
      return NULL;
   }
};

// ---------------------------------------------------------------

// The application object
class MyApp: public wxApp
{
   virtual bool OnInit();
   virtual int  OnExit();

private:
  CoreThread* corethread; // does audio i/o and DSP script processing
};

// ---------------------------------------------------------------

//class MySpectrumFrame; // predeclaration
//class MyOscilloscopeFrame;

// ---------------------------------------------------------------

// The main frame class for our application
class MyFrame: public wxFrame, GraphFrameCloseReceiver
{
public:

   MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size, int fontsize);

   void loadfile(const wxString filename);
   void setsource(int);

   void DoOpenWaveFile(const wxString& filename);

   void GFrameCloseNotify(MyGraphFrame *isClosing); // mygraphframe calls this when it's closed.

private:
   void OnQuit(wxCommandEvent& event);
   void OnAbout(wxCommandEvent& event);
   void OnHelp(wxCommandEvent& event);
   void OnHelpHomepage1(wxCommandEvent& event);
   void OnHelpHomepage2(wxCommandEvent& event);
   void OnOpen(wxCommandEvent& event);
   void OnReload(wxCommandEvent& event);
   void OnSave(wxCommandEvent& event);
   void OnSaveAs(wxCommandEvent& event);
   void OnClear(wxCommandEvent& event);
   void OnShowSpectrum(wxCommandEvent& event);
   void OnShowOscilloscope(wxCommandEvent& event);
   void OnDisplaysliderEvent(wxCommandEvent &event);

   void OnDCOffsetChanged(wxCommandEvent& event);
   void OnInputChanged(wxCommandEvent& event);

   void OnRunbutton(wxCommandEvent& event);
   void OnStopbutton(wxCommandEvent& event);
   void OnTextchanged(wxCommandEvent& event);
   void OnTimer(wxTimerEvent& event);

   void OnOpenWaveFile(wxCommandEvent& event);


   DECLARE_EVENT_TABLE()

   std::vector<DisplaySlider*> paramsliders;
   std::vector<DisplaySlider*> freqsliders;
   std::vector<int> inputidentifiers;

   wxRadioBox *inputselector;
   wxBook     *inputbook;

   wxStaticText *waveinfotext;  // wave info
   wxCheckBox *removedc;

   wxTextCtrl *text;			// program editbox
   wxFont     textfont;   // the font used for the program editor

   wxToggleButton *spectrumbutton;	// Show or hides the spectrumplot window when clicked.
   MyGraphFrame *specframe;

   wxToggleButton *oscilloscopebutton;	// Show or hides the oscilloscopeplot window when clicked.
   MyGraphFrame *scopeframe;

   wxVUMeter *vu_left;
   wxVUMeter *vu_right;

   wxTimer m_timer;         // timer used for updating parts of the GUI as a function of output from the core thread

   wxPanel    *panel;       // the topmost panel object
   wxBoxSizer *topsizer;    // the topmost sizer for the wxFrame

   void UpdateWindowSize(); // resize the window to fit the controls

   void settextbackgroundcolour(const wxColour& colour);
   void updatetitle(void);  // changes the program's title to include the file name
   void recompile(void);
   void setslider_core(int,float);
   void OpenHelpWindow(const wxString &contents);
   void do_setsource(int i);

   int backgroundgreen;
   wxString filename;
   wxStaticText *statustext;
   float spec_l[NFFT];
   float spec_r[NFFT];
};


#endif 
