

#include "mutex.h"

Mutex::Mutex()
{
#ifdef WIN32
  InitializeCriticalSection(&cs);
#else
  pthread_mutex_init(&my_mutex, NULL);
#endif
}

Mutex::~Mutex()
{
#ifdef WIN32
  DeleteCriticalSection(&cs);
#else
  pthread_mutex_destroy(&my_mutex);
#endif
}

void Mutex::Lock()
{
#ifdef WIN32
  EnterCriticalSection(&cs);
#else
  pthread_mutex_lock(&my_mutex);
#endif
}

void Mutex::Unlock()
{
#if WIN32
  LeaveCriticalSection(&cs);
#else
  pthread_mutex_unlock(&my_mutex);
#endif
}

// -----------------------------------------------------------------

MutexLocker::MutexLocker(Mutex *_m) : m(_m)
{
  m->Lock();
}

MutexLocker::~MutexLocker()
{
  m->Unlock();
}
