/********************************************************************
  BasicDSP
  Copyright 2006-2007 
  Pieter-Tjerk de Boer PA3FWM pa3fwm@amsat.org
  Niels Moseley PE1OIT n.a.moseley@alumnus.utwente.nl
  License: GPLv2
********************************************************************/

#include "spectrumplot.h"
#include "math.h"
#include "core.h"
#include "vu_meter.h"     // sigh, for myYELLOW_PEN
#include <vector>


BEGIN_EVENT_TABLE(wxSpectrumplot, wxControl)
   EVT_PAINT(wxSpectrumplot::OnPaint)
   EVT_ERASE_BACKGROUND(wxSpectrumplot::OnEraseBackGround)
   EVT_SIZE(wxSpectrumplot::OnSize)
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(wxSpectrumplot, wxControl)

wxSpectrumplot::~wxSpectrumplot()
{
  delete tempbitmap;
  dc_background.SelectObject(wxNullBitmap);
  delete background;
}

bool wxSpectrumplot::Create(wxWindow *parent, wxWindowID id, const wxString &label,
	const wxString &xf,
	const wxString &xfk,
	const wxPoint& pos,
	const wxSize& size
	)
{
	if (!wxControl::Create(parent, id, pos, size, wxSIMPLE_BORDER))
		return false;

	SetLabel(label);
	xfmt = xf;
	xfmtk = xfk;
	return true;
}

void wxSpectrumplot::SetLevel(float *l,float *r)
{
  memcpy(yellow, l, sizeof(float)*n);
  memcpy(cyan, r, sizeof(float)*n);
  Refresh();
}

void wxSpectrumplot::SetMinMax(float dbmi,float dbma, float fmi, float fma, float xz)
{
   if (dbmin!=dbmi || dbmax!=dbma || fmin!=fmi || fmax!=fma) backgroundredrawneeded=1;
   int refresh = (backgroundredrawneeded || xzoom!=xz);
   dbmin=dbmi;
   dbmax=dbma;
   fmin=fmi;
   fmax=fma;
   xzoom=xz;
   if (refresh) Refresh();
}

int wxSpectrumplot::db2pix(float db)
{
   return static_cast<int>((dbmax - db) / (dbmax-dbmin) * height);
} 


void wxSpectrumplot::OnSize(wxSizeEvent& event)
{
   if (background) 
   {
     dc_background.SelectObject(wxNullBitmap);
     delete background;
     background=NULL;
   }
   int cwidth  = GetClientRect().GetWidth();
   int cheight = GetClientRect().GetHeight();
   tempbitmap = new wxBitmap(cwidth, cheight, -1);
   background = new wxBitmap(cwidth, cheight, -1);
   dc_background.SelectObject(*background);

   backgroundredrawneeded=true;
   Refresh();
}


void wxSpectrumplot::OnPaint(wxPaintEvent& event)
{
   wxPaintDC realdc(this);
   wxMemoryDC dc(&realdc);
   dc.SelectObject(*tempbitmap);

   wxRect rect = GetClientRect();
   height = rect.GetHeight();
   width = rect.GetWidth();

   wxString stringbuf;

   if (backgroundredrawneeded) 
   {
      backgroundredrawneeded=false;

      // erase everything
      dc_background.SetPen(*wxBLACK_PEN);
      dc_background.SetBrush(*wxBLACK_BRUSH);
      dc_background.DrawRectangle(0, 0, rect.GetWidth(), rect.GetHeight());

      // draw the dB scale
      dc_background.SetFont(*wxSMALL_FONT);
      dc_background.SetPen(*wxGREY_PEN);
      dc_background.SetTextForeground(wxColour(_T("white")));
      wxString stringbuf;
      if (dbmax-dbmin>10) {
         for(int i=0; i>=dbmin; i-=10) {
            int ypos = db2pix(i);
            dc_background.DrawLine(0, ypos, width-1, ypos);
            stringbuf.Printf(_T("%ddB"),i);
            dc_background.DrawText(stringbuf, 1, ypos);
         }
      } else {
         // treatment of this case is a hack, not generally valid yet
         const float ypos1[]={-0.1,0,0.1, 2e30};
         const float ypos2[]={-1,0,1, 2e30};
         const float *yp=ypos1;
         if (dbmax-dbmin>=2) yp=ypos2;
         while (*yp<1e30) {
            int ypos=db2pix(*yp);
            dc_background.DrawLine(0, ypos, width-1, ypos);
            stringbuf.Printf(_T("%g"),*yp);
            dc_background.DrawText(stringbuf, 1, ypos);
            yp++;
         }
      }

      // draw the frequency scale
      int step = 100;
      const int steps[] = {1,2,5,10,20,50,100,200,500,1000,2000,5000,10000,20000,50000,0};
      const int *sp;
      wxCoord twidth, theight;
      GetTextExtent(_T("8"), &twidth, &theight, NULL, NULL, wxSMALL_FONT);
      int maxlabels = width/(twidth*8);
      sp=steps;
      while ((fmax-fmin)/sp[0] > maxlabels && sp[1]>0) sp++;
      step = *sp;
      for (int i=-step*((int)(-fmin/step+1000)-1000); i<=fmax; i+=step) {
         int x = (int)(0.5+(i-fmin)/(fmax-fmin)*width);
         dc_background.DrawLine(x, 0, x, height-1);
         if (step>=1000) stringbuf.Printf(xfmtk,i/1000);
         else            stringbuf.Printf(xfmt,i);

         GetTextExtent(stringbuf, &twidth, &theight, NULL, NULL, wxSMALL_FONT);

         if (x-twidth/2 < 0)          dc_background.DrawText(stringbuf, 0, height-theight);
         else if (x+twidth/2 > width) dc_background.DrawText(stringbuf, width-twidth, height-theight);
         else                         dc_background.DrawText(stringbuf, x-twidth/2, height-theight);
      }
   }

   dc.Blit(0,0, rect.GetWidth(), rect.GetHeight(), &dc_background, 0, 0, wxCOPY, true);

   dc.SetPen(*myYELLOW_PEN);
   std::vector<wxPoint> list(n*2);
   for (int i=0;i<n;i++) {
      list[i].x = (int)(0.5+(float)i/n*width*xzoom);
      int y = db2pix(yellow[i]);
      if (y<-10) y=-10;
      if (y>height+10) y=height+10;
      list[i].y = y;
   }
   dc.DrawLines(n,&(list[0]));

   dc.SetPen(*wxCYAN_PEN);
   for (int i=0;i<n;i++) {
      list[i].x = (int)(0.5+(float)i/n*width*xzoom);
      int y = db2pix(cyan[i]);
      if (y<-10) y=-10;
      if (y>height+10) y=height+10;
      list[i].y = y;
   }
   dc.DrawLines(n,&(list[0]));
   dc.SelectObject(wxNullBitmap); // we must do this to unselect the tempbitmap before we can use it!
   realdc.DrawBitmap(*tempbitmap, 0, 0);
}

void wxSpectrumplot::OnEraseBackGround(wxEraseEvent& event)
{
   // do nothing.. we don't want to erase the background!
};


wxSize wxSpectrumplot::DoGetBestSize() const
{
	return wxSize(GetRect().GetWidth(), 100);
}
