/********************************************************************
  BasicDSP
  Copyright 2006-2007 
  Pieter-Tjerk de Boer PA3FWM pa3fwm@amsat.org
  Niels Moseley PE1OIT n.a.moseley@alumnus.utwente.nl
  License: GPLv2
********************************************************************/

#ifndef __vu_meter_h
#define __vu_meter_h

#include "wx/wx.h"

class wxVUMeter : public wxControl
{
	DECLARE_DYNAMIC_CLASS(wxVUMeter)
	DECLARE_EVENT_TABLE()

public:
	virtual ~wxVUMeter();

	wxVUMeter()
	{
		Init();
	};

	wxVUMeter(wxWindow *parent, wxWindowID id, 
		const wxString &label,
		const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize) 
	{
		Init();
		Create(parent, id, label, pos, size);
	};

	bool Create(wxWindow *parent, wxWindowID id, 
		const wxString &label,
		const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize);

	void Init() { 
		level = 1e-40; 
    peaklevel = level;
    peakcounter = 0;
		dbmin = -54;
		dbmax = 3;
		background = NULL;
    tempbitmap = NULL;
	};

	wxSize DoGetBestSize() const;

   void SetLevel(float l);

private:
   wxBitmap *tempbitmap;     // temporary bitmap so we can avoid using wxBufferedPaintDC
   wxBitmap *background;     // actually the (non-changing) foreground, namely the scale etc.
   wxMemoryDC dc_background;

   int db2pix(float db);
   float level;        // the current level shown on the VU meter
   float peaklevel;    // the current peaklevel shown on the VU meter
   int peakcounter;    // number of updates still to go before we reset the peaklevel
   float dbmin;        // minimum level shown on the VU meter (in dB)
   float dbmax;        // maximum level shown on the VU meter (in dB)

   // Event handlers
   void OnPaint(wxPaintEvent& event);
   void OnEraseBackground(wxEraseEvent& event);
   void OnSize(wxSizeEvent& event);

};

extern wxPen *myYELLOW_PEN;

#endif

