/********************************************************************

  Wav streamer rev3.
  Code by N.A. Moseley 
  Copyright 2006-2007

  Note: only stereo files using PCM or IEEE floats are supported!
  supports 16,24,32 bit PCM and 32 bit float.
  stereo only!!!
  License: GPLv2

********************************************************************/

#ifndef wav_streamer_h
#define wav_streamer_h

#include <wx/string.h>
#include <wx/stream.h>
#include <wx/thread.h>
#include <wx/wfstream.h>

// ----------------------------------------------------------

#pragma pack(push)
#pragma pack(1)

typedef struct {
//  char           chunkID[4];  // 'fmt '
//  long           chunkSize;   // size of the following data (in bytes)

  short          wFormatTag;  // should be '1' for uncompressed PCM, or '3' for uncompressed IEEE float data.
  unsigned short wChannels;
  unsigned long  dwSamplesPerSec;
  unsigned long  dwAvgBytesPerSec;
  unsigned short wBlockAlign;
  unsigned short wBitsPerSample;

/* Note: there may be additional fields here, depending upon wFormatTag. */

} WavFormatChunk;

//typedef struct {
//  char           chunkID[4];  // 'data'
//  long           chunkSize;

//  unsigned char  *waveformData;
//} WavDataChunk;

#pragma pack(pop)

// ----------------------------------------------------------

class WavStreamer
{
public:
  WavStreamer();
  virtual ~WavStreamer();

  /** Opens a file for reading
      @return error code. 0 = ok, -1 = invalid format, -2 = cannot open file
  */
  int OpenFile(const wxString &filename);

  /** Fill a float buffer with L/R stereo samples
    @param stereo_buffer a pointer to a float buffer
    @param stereo_samples number of stereo samples to write to buffer
    note: total bytes written to the buffer is sizeof(float)*2*stereo_samples.
    if no file was opened previously, the buffer will be filled with zeroes.
  */
#ifdef WIN32
  void __fastcall FillBuffer(float *stereo_buffer, int stereo_samples);
#else
  void FillBuffer(float *stereo_buffer, int stereo_samples);
#endif

  /** returns the filename of the currently loaded file. */
  wxString GetFilename() {wxMutexLocker lock(mutex); return my_filename;};

  /** fills output with format info
    @param output a reference to a WavFormatChunk to be filled
    @return true if operation was successful
  */
  bool GetFormat(WavFormatChunk &output) const;

protected:
  wxMutex         mutex;            // mutex for protecting loading & buffer filling.

  bool FindChunk(const char ID[4]);

  /** ReadSamples fills temp_buffer with data.
      Actual byte-count depends on the sample size (16 bit, 24 bit, 32 bit or float)
  */
  int  ReadRawData(int RequestedSamples); 

  wxFileInputStream *wavefile;      // the file to be used
  wxString          my_filename;

  unsigned long  riff_size;
  unsigned long  playback_offset;   // playback offset (into file)
  unsigned long  playback_start;    // start of audio data within .wav file
  unsigned long  playback_end;      // end of audio data within .wav file

  char           chunk_type[4];     // type of chunk
  long           chunk_size;        // size of chunk in bytes

  WavFormatChunk  myformat;

  int            sample_index;     // where to start reading the temp_buffer (in mono sample offset) 0..8191
  void           *temp_buffer;     // holds temporary file data, has 4096 stereo sample entries.
};

#endif  //Sentry
