/*

	BasicDSP - by Pieter-Tjerk de Boer
	Win32 port by Niels Moseley PE1OIT (c) 2006

*/

#include <windows.h>
#include <conio.h>
#include <stdio.h>
#include <string>
#include "core.h"
#include "portaudio.h"

/* global audio error buffer -- ugly, but what the heck.. */
char AudioError[1024];

PaStream *stream = NULL;
PaDeviceIndex my_indevice = 0;
PaDeviceIndex my_outdevice = 0;
bool quit_flag = false;

static int paProcessCallback( const void *inputBuffer, void *outputBuffer,
                           unsigned long framesPerBuffer,
                           const PaStreamCallbackTimeInfo* timeInfo,
                           PaStreamCallbackFlags statusFlags,
                           void *userData )
{
    /* Cast data passed through stream to our structure. */
    //paTestData *data = (paTestData*)userData; 
    short *buf_in  = (short*)inputBuffer;
	short *buf_out = (short*)outputBuffer;

	process_samples(buf_in,buf_out, framesPerBuffer);	// do your audio processing stuff!

    return 0;
}

//// -------------------------------------------------

bool CreatePortaudioStream(PaDeviceIndex in_device, PaDeviceIndex out_device)
{
  PaStreamParameters inputParameters;
  PaStreamParameters outputParameters;

  inputParameters.device = in_device;
  inputParameters.channelCount = 2;
  inputParameters.sampleFormat = paInt16;
  inputParameters.hostApiSpecificStreamInfo = NULL;
  inputParameters.suggestedLatency = 0.1;

  outputParameters.device = out_device;
  outputParameters.channelCount = 2;
  outputParameters.sampleFormat = paInt16;
  outputParameters.hostApiSpecificStreamInfo = NULL;
  outputParameters.suggestedLatency = 0.1;

  PaError err = Pa_OpenStream(&stream, 
    &inputParameters, &outputParameters, samplerate, 256, paNoFlag, paProcessCallback, NULL);

  if( err != paNoError ) goto sound_error;

	err = Pa_StartStream( stream );
    if( err != paNoError ) goto sound_error;
	
	memset(AudioError,0,1024);	// clear error text!
	return true;

sound_error:
	const char *error_text = Pa_GetErrorText(err);
	std::string error_str("Audio: ");
	std::string error_txt(error_text);
	error_str += error_txt;
	strcpy(AudioError,error_txt.c_str()); // set error text!
	return false;
}

//// -------------------------------------------------

bool KillPortaudioStream()
{
	PaError err = Pa_StopStream( stream );
    if( err != paNoError ) goto sound_error;

	return true;

sound_error:
	return false;
}

//// -------------------------------------------------

void coremain(void)
{
	bool audio_ok = false;
	float last_samplerate = samplerate;

	/* Open a portaudio stream */
	PaError err = Pa_Initialize();
	if (err != paNoError ) goto sound_error;

  // search for a directsound device
  const   PaHostApiInfo *apiInfo;
  int numApi = Pa_GetHostApiCount();
  my_indevice = 0;
  my_outdevice = 0;
  for(int i=0; i<numApi; i++)
  {
    apiInfo = Pa_GetHostApiInfo(i);
    if (apiInfo->type == paDirectSound)
    {
      my_indevice = apiInfo->defaultInputDevice;
      my_outdevice  = apiInfo->defaultOutputDevice;
    }
  }
  // if no device found, search for MME devices.
  if ((my_indevice == 0) || (my_outdevice == 0))
  {
    for(int i=0; i<numApi; i++)
    {
      apiInfo = Pa_GetHostApiInfo(i);
      if (apiInfo->type == paMME)
      {
        my_indevice = apiInfo->defaultInputDevice;
        my_outdevice  = apiInfo->defaultOutputDevice;
      }
    }
  }

	if (CreatePortaudioStream(my_indevice, my_outdevice)==false)
	{
		// error opening the audio stream
		goto sound_error;
	}

	audio_ok = true;

	while(quit_flag == false)
	{
		if (last_samplerate != samplerate)
		{
			//close the previous stream and open a new one with the updated sample-rate.
			if (audio_ok) KillPortaudioStream();
			if (CreatePortaudioStream(my_indevice, my_outdevice))
				audio_ok = true;
			else
				audio_ok = false;
			last_samplerate = samplerate;
		}
		Sleep(500);		// wait 500msec before checking stuff.
	}

	KillPortaudioStream();

	/* Exit the program */
	err = Pa_Terminate();
	if( err != paNoError )
	   printf(  "PortAudio error: %s\n", Pa_GetErrorText(err));

	return;

sound_error:		
	printf("PortAudio error: %s\n", Pa_GetErrorText(err));
	Pa_Terminate();	// Call terminate just to be safe.
	return;
}

